require 'spec_helper'

describe Scripting::Commands do
  it "should symbolize and downcase the command name" do
    Scripting::Commands::Command.new(nil, :testing).name.should == :testing
    Scripting::Commands::Command.new(nil, :Testing).name.should == :testing
    Scripting::Commands::Command.new(nil, 'testing').name.should == :testing
    Scripting::Commands::Command.new(nil, "Testing").name.should == :testing
    Scripting::Commands::Command.new(nil, "TESTING").name.should == :testing
  end
end

describe Scripting::Commands::Command do
  before :each do
    @stdout = ''
    STDOUT.stub(:write) { |*args| @stdout << args.join }

    @stderr = ''
    STDERR.stub(:write) { |*args| @stderr << args.join }

    @app = Scripting::create_application do
      plugin Scripting::Commands

      def some_method; :some_method_return_value; end
      options { some_option :some_option }
      
      command :answer do
        description "The answer to life, the universe, and everything"
        work { puts 42 }
      end

      command :echo do
        description "Echo args"
        work { |*args| puts args.join(" ") }
      end

      command :add do
        description "Add two numbers"
        work { |a,b| puts a.to_f + b.to_f }
      end

      command :context do
        description "Print self.inspect"
        work { print self.object_id }
      end

      command :call_some_method do
        description "Call the some_method method"
        work { print some_method.to_s }
      end

      command :print_some_option do
        description "Print some_option value"
        work { print options.some_option.to_s }
      end
    end
  end

  it "should populate the help text with details for each command" do
    @app.help!
    @stderr.should include("answer: The answer to life, the universe, and everything")
    @stderr.should include("echo: Echo args")
    @stderr.should include("add: Add two numbers")
  end

  it "should run the specified command" do
    @app.run! 'answer'
    @stdout.should == "42\n"
  end

  it "should send arguments to the specified command" do
    @app.run! %w{add 4 8}
    @stdout.should == "12.0\n"
  end

  it "should run the specified command as well as other work items" do
    @app.work { puts "Hello world!" }
    @app.run! %w{echo to infinity and beyond!}
    @stdout.should == "to infinity and beyond!\nHello world!\n"
  end

  it "should run in the context of the application" do
    @app.run! 'context'
    @stdout.should == @app.object_id.to_s
  end

  it "should call local application methods directly" do
    @app.run! 'call_some_method'
    @stdout.should == @app.some_method.to_s
  end

  it "should have direct access to options" do
    @app.run! 'print_some_option'
    @stdout.should == @app.options.some_option.to_s
  end
end
