require 'spec_helper'

describe Scripting::Pluggable do
  it "should apply both ClassMethods and InstanceMethods" do
    module PluginA
      module ClassMethods
        def my_method; :class_method_result; end
      end
      module InstanceMethods
        def my_method; :instance_method_result; end
      end
    end

    exemplar = Scripting::Application.new
    exemplar.plugins.should be_empty
    lambda { exemplar.class.my_method }.should raise_error(NoMethodError)
    lambda { exemplar.my_method }.should raise_error(NoMethodError)

    with_plugin = Scripting::create_application do
      plugin PluginA
    end
    with_plugin.plugins.should == [PluginA]
    with_plugin.class.my_method.should == :class_method_result
    with_plugin.my_method.should == :instance_method_result

    without_plugin = Scripting::create_application do
    end
    without_plugin.plugins.should be_empty
    lambda { without_plugin.class.my_method }.should raise_error(NoMethodError)
    lambda { without_plugin.my_method }.should raise_error(NoMethodError)
  end

  it "should apply only ClassMethods if InstanceMethods are undeclared" do
    module PluginB
      module ClassMethods
        def my_method; :class_method_result; end
      end
    end

    app = Scripting::create_application do
      plugin PluginB
    end
    app.plugins.should == [PluginB]
    app.class.my_method.should == :class_method_result
    lambda { app.my_method }.should raise_error(NoMethodError)
  end

  it "should apply only InstanceMethods if ClassMethods are undeclared" do
    module PluginC
      module InstanceMethods
        def my_method; :instance_method_result; end
      end
    end

    app = Scripting::create_application do
      plugin PluginC
    end
    app.plugins.should == [PluginC]
    lambda { app.class.my_method }.should raise_error(NoMethodError)
    app.my_method.should == :instance_method_result
  end

  it "should run class_init and instance_init of they are declared" do
    module PluginD
      def self.class_init(base, *args)
        base.class_eval do
          def self.foo; 0xdeadbeef; end
        end
      end

      def self.instance_init(instance, *args)
        instance.instance_eval do
          def foo; 0x8ba11; end
        end
      end
    end

    app = Scripting::create_application do
      plugin PluginD
    end

    app.plugins.should == [PluginD]
    app.class.foo.should == 0xdeadbeef
    app.foo.should == 0x8ba11
  end

  it "should allow multiple plugins" do
    module PluginE
    end

    module PluginF
    end

    app = Scripting::create_application do
      plugin PluginE
      plugin PluginF
    end

    app.plugins.should == [PluginE, PluginF]
  end
end